package cz.muni.pa165.rating.rest;

import cz.muni.pa165.core.api.rating.MovieRatingDto;
import cz.muni.pa165.rating.data.model.RatingFilter;
import cz.muni.pa165.rating.facade.MovieRatingFacade;
import io.micrometer.observation.Observation;
import io.micrometer.observation.ObservationRegistry;
import io.swagger.v3.oas.annotations.Operation;
import jakarta.validation.Valid;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(path = "${rest.path.ratings}")
public class MovieRatingRestController {

    private final MovieRatingFacade movieRatingFacade;
    private final ObservationRegistry observationRegistry;

    @Autowired
    public MovieRatingRestController(
        MovieRatingFacade movieRatingFacade,
        ObservationRegistry observationRegistry
    ) {
        this.movieRatingFacade = movieRatingFacade;
        this.observationRegistry = observationRegistry;
    }

    @Operation(summary = "Get all movie ratings")
    @GetMapping
    @ResponseStatus(HttpStatus.OK)
    public Page<MovieRatingDto> getMovieRatings(
        @ParameterObject RatingFilter filter,
        @ParameterObject Pageable pageable
    ) {
        final Page<MovieRatingDto> page = Observation
            .createNotStarted("rating.getMovieRatings", observationRegistry)
            .lowCardinalityKeyValue(
                "page",
                String.valueOf(pageable.getPageNumber())
            )
            .lowCardinalityKeyValue(
                "size",
                String.valueOf(pageable.getPageSize())
            )
            .highCardinalityKeyValue("highTag", "highValue")
            .observe(() -> movieRatingFacade.findAll(filter, pageable));
        return page;
    }

    @Operation(summary = "Get movie rating by id")
    @GetMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.OK)
    public MovieRatingDto getMovieRatingById(@PathVariable long id) {
        final MovieRatingDto dto = Observation
            .createNotStarted("rating.getMovieRatingById", observationRegistry)
            .lowCardinalityKeyValue("id", String.valueOf(id))
            .highCardinalityKeyValue("highTag", "highValue")
            .observe(() -> movieRatingFacade.findById(id));
        return dto;
    }

    @Operation(summary = "Create new movie rating")
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    public MovieRatingDto createMovieRating(
        @Valid @RequestBody MovieRatingDto movieRating
    ) {
        final MovieRatingDto dto = Observation
            .createNotStarted("rating.createMovieRating", observationRegistry)
            .lowCardinalityKeyValue(
                "movieId",
                String.valueOf(movieRating.getMovieId())
            )
            .lowCardinalityKeyValue(
                "userId",
                String.valueOf(movieRating.getUserId())
            )
            .highCardinalityKeyValue("highTag", "highValue")
            .observe(() -> movieRatingFacade.create(movieRating));
        return dto;
    }

    @Operation(summary = "Update movie rating")
    @PutMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.OK)
    public MovieRatingDto updateMovieRating(
        @PathVariable long id,
        @RequestBody MovieRatingDto movieRating
    ) {
        final MovieRatingDto dto = Observation
            .createNotStarted("rating.updateMovieRating", observationRegistry)
            .lowCardinalityKeyValue("id", String.valueOf(id))
            .lowCardinalityKeyValue(
                "movieId",
                String.valueOf(movieRating.getMovieId())
            )
            .lowCardinalityKeyValue(
                "userId",
                String.valueOf(movieRating.getUserId())
            )
            .highCardinalityKeyValue("highTag", "highValue")
            .observe(() -> movieRatingFacade.update(id, movieRating));
        return dto;
    }

    @Operation(summary = "Delete movie rating")
    @DeleteMapping(path = "${rest.path.id}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public void deleteMovieRating(@PathVariable long id) {
        Observation
            .createNotStarted("rating.deleteMovieRating", observationRegistry)
            .lowCardinalityKeyValue("id", String.valueOf(id))
            .highCardinalityKeyValue("highTag", "highValue")
            .observe(() -> movieRatingFacade.delete(id));
    }
}
